-module(kr).
-compile(export_all).
-import(vigenere).
-include_lib("eunit/include/eunit.hrl").

encrypt(Message, Key, Salt) ->
    encrypt(Message, Key, Salt, {$A, $Z}).

encrypt(Message, Key, Salt, Range) ->
    Cipher = vigenere:encrypt(Message, Key, Range),
    put_salt(Cipher, Salt, []).

decrypt(SaltedCipher, Key, Salt) ->
    decrypt(SaltedCipher, Key, Salt, {$A, $Z}).

decrypt(SaltedCipher, Key, Salt, Range) ->
    Cipher = put_salt(SaltedCipher, -Salt, []),
    vigenere:decrypt(Cipher, Key, Range).

put_salt([], _, SaltedCipher) ->
    lists:reverse(SaltedCipher);
put_salt([C|Cipher], Salt, SaltedCipher) ->
    put_salt(Cipher, Salt, [apply_salt(C, Salt)|SaltedCipher]).

apply_salt(Code, Salt) ->
    apply_salt(binary:encode_unsigned(Code), Salt, 0).

apply_salt(<<C:8, Code/bytes>>, Salt, Acc) ->
    H = ((C bsr 4) + Salt) bsl 4,      % add salt to higher byte
    L = ((C band 15) + Salt) band 15,  % add salt to lower byte
    apply_salt(Code, Salt, (Acc bsl 8) + (H bor L));
apply_salt(<<>>, _, Acc) -> Acc.

%%----------------------------------------------------------------------
%% Tests
%%----------------------------------------------------------------------

encrypt_test_() ->
    [?_assertMatch([127,139,121,114,131,137,120,121,133,113,123,133], encrypt("ATTACKATDAWN", "LEMON", 3))].

decrypt_test_() ->
    [?_assertMatch("ATTACKATDAWN", decrypt([127,139,121,114,131,137,120,121,133,113,123,133], "LEMON", 3))].

apply_salt_test_() ->
    [?_assertMatch(270, apply_salt(253, 1)),
     ?_assertMatch(4626, apply_salt(257, 1)),
     ?_assertMatch(257, apply_salt(4626, -1)),
     ?_assertMatch(17, apply_salt(0, 1))].

acceptance() ->
    M = unicode:characters_to_list(list_to_binary("мой холодный год плодородный")),
    K = unicode:characters_to_list(list_to_binary("природа")),
    S = 3, % 0011b
    C = encrypt(M, K, S, {1072, 1103}), % {а, я}
    D = decrypt(C, K, S, {1072, 1103}), % {а, я}
    erlang:display(M),
    erlang:display(K),
    erlang:display(C),
    erlang:display(D).
